/*
 * main.c
 *
 * Author : Anders Zeijlon
 */ 


//==============================================================================
//--------------------------- Include Files ------------------------------------
//==============================================================================
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include "PcUart.h"
#include "Azande.h"

//==============================================================================
//--------------------------- Public Defines, Typedef --------------------------
//==============================================================================
#define SETBIT(ADDRESS,BIT) (ADDRESS |= (1<<BIT))


/* Variables */
//==============================================================================
//--------------------------- Variables  ---------------------------------------
//==============================================================================
volatile uint8_t execute;
static uint8_t init;
static int32_t counter = 0;
static int32_t waterlevel = 0;
static double percentage;
static double airtemperature;
static uint8_t redLedIsOn = 0;


//==============================================================================
//--------------------------- Azande Macros ------------------------------------
//==============================================================================

define_target_info(targetInfo, "Atmel Demo1", "version 1.0.1", "2018-02-05", "Testing")

define_void_command(cmdRestartSystem, "Restart System", 0, RestartSystem);
define_bool_command(cmdSetRedLed, "Set Red LED", 4, setRedLed, "ON", "OFF" );
define_int_command(cmdSetDac1, "Set DAC1", 2, SetDac1, "steps", 0, 1023);
define_int_command(cmdSetCounter, "Set Counter", 11, SetCounter, , , );
define_double_command(cmdSetBacklightVoltage, "Set Backlight Voltage", 0, SetBacklightVoltage, "V", 0.0, 3.3 );
define_double_command(cmdSetPercentage, "Set Percentage", 10, SetPercentage, , , );
define_text_command(cmdSetMsgOnDisplay, "Set Msg On Display", 12, SetMsgOnDisplay, 32 );

define_enum_command(cmdSetLedColor, "Set LED Color", 19, SetLedColor,   define_enum_item(0, "Green")\
define_enum_item(1, "Red")\
define_enum_item(-25, "Blue")\
define_enum_item(3, "Yellow")\
define_enum_item(23, "White"))

define_bool_event(eventButtonAState, "Button-A State", 6, "Activated", "Not Activated" );
define_int_event(eventWaterLevel, "Water Level", 4, "cm", -30, 30, "D");
define_int_event(eventCounter, "Counter", 11, , , , );
define_double_event(eventAirTemperature, "Air Temperature", 4, "C", 0.0, 90.0, "F2");
define_double_event(eventPercentage, "Percentage", 10, , , , );
define_text_event(eventMsgFromUser, "Msg From User", 12, 64);


//==============================================================================
//--------------------------- Local Function Prototypes ------------------------
//==============================================================================

static void initTimer0(void);
static void ExecuteWaterLevel(void);
static void ExecuteAirTemperature(void);

//==============================================================================
//--------------------------- Main ---------------------------------------------
//==============================================================================

int  main(void)
{
	uint8_t t1ms = 0;
	uint8_t t10ms = 0;
	uint8_t t100ms = 0;
	uint8_t t1s = 0;

	PcUart_init();
	Azande__Init(targetInfo);
	initTimer0();
	
	init = 1;
	sei();      	//global interrupt enable

	Azande__AddCommand(cmdRestartSystem);
	Azande__AddCommand(cmdSetRedLed);
	Azande__AddCommand(cmdSetDac1);
	Azande__AddCommand(cmdSetCounter);
	Azande__AddCommand(cmdSetLedColor);
	Azande__AddCommand(cmdSetBacklightVoltage);
	Azande__AddCommand(cmdSetPercentage);
	Azande__AddCommand(cmdSetMsgOnDisplay);
	Azande__AddEvent(eventButtonAState);
	Azande__AddEvent(eventWaterLevel);
	Azande__AddEvent(eventCounter);
	Azande__AddEvent(eventAirTemperature);
	Azande__AddEvent(eventPercentage);
	Azande__AddEvent(eventMsgFromUser);

	for(;;)
	{
		
		if (execute == 0x01) //1 ms
		{
			execute = 0x00;
			// Execute every 1 ms ===========================================================================
			// ==============================================================================================			
			Azande__ReadStream();
			// ==============================================================================================
			// ==============================================================================================
			
			t1ms++;
			if (t1ms == 10)	//10 ms
			{
				t1ms = 0;
				t10ms++;
			// Execute every 10 ms ==========================================================================
			// ==============================================================================================
			// ==============================================================================================
			// ==============================================================================================
				if (init == 1)
				{
						init = 0;
				}
				
				if (t10ms == 10)//100ms
				{
					t10ms = 0;
					t100ms++;
			// Execute every 100 ms =========================================================================
			// ==============================================================================================
					ExecuteWaterLevel();
					ExecuteAirTemperature();
			// ==============================================================================================
			// ==============================================================================================
								
					if (t100ms == 10)//1 s
					{
						t100ms = 0;
			// Execute every 1 s ============================================================================
			// ==============================================================================================

			// ==============================================================================================
			// ==============================================================================================

						t1s++;
						if (t1s == 10)//10 s
						{
							t1s = 0;
			// Execute every 1 s ============================================================================
			// ==============================================================================================

			// ==============================================================================================
			// ==============================================================================================

						}
					}
				}
			}
		}
	}
}



//==============================================================================
//--------------------------- Azande Command Functions ----------------------------------
//==============================================================================


void setRedLed(uint8_t state)
{
	// digitalWrite(led, state);
	redLedIsOn = state;
}

void RestartSystem(void)
{
}

void SetDac1(long steps)
{
}

void SetLedColor(int32_t colorId)
{
}

void SetCounter(int32_t count)
{
	counter = count;
	Azande__SendEventInt32(eventCounter, counter);
}

void SetBacklightVoltage(double voltage)
{
}

void SetPercentage(double per)
{
	percentage = per;
	Azande__SendEventDouble(eventPercentage, percentage);
}

void SetMsgOnDisplay(char* txt, unsigned int length)
{
	char str[64];
	memcpy(str, txt, length);
	str[length] = 0;
	Azande__SendEventText(eventMsgFromUser, str);
}


//==============================================================================
//--------------------------- Local Functions ----------------------------------
//==============================================================================

static void initTimer0(void)
{
	OCR0A = 30;	//Clear timer when it reaches this value
	SETBIT(TCCR0A,WGM01);//Timer 0 Clear-Timer on Compare (CTC)
	SETBIT(TCCR0B,CS02);
	SETBIT(TIMSK0,OCIE0A);//Enable compare A interrupt for Timer0
}


ISR (TIMER0_COMPA_vect)
{
	execute = 0x01; //Exekvera
}

static void ExecuteWaterLevel(void)
{
	++waterlevel;
	if (waterlevel > 30)
	{
		waterlevel = -30;
	}
	Azande__SendEventInt32(eventWaterLevel, waterlevel);
}

static void ExecuteAirTemperature(void)
{
      airtemperature += 0.2;
      if (airtemperature > 90.0)
      {
	      airtemperature = 0.0;
      }
      Azande__SendEventDouble(eventAirTemperature, airtemperature);
}
