/*
 * Azande.h
 *
 * Author : Anders Zeijlon
 */ 

#ifndef AZANDE_H_
#define AZANDE_H_



//==============================================================================
//--------------------------- Include Files ------------------------------------
//==============================================================================
#include <stdint.h>
#include "AzandeSystemConfig.h"
#include "AzandeUserMacros.h"



//==============================================================================
//--------------------------- Public Defines, Typedef --------------------------
//==============================================================================

typedef enum
{
	AZANDE_COMMAND_TYPE_VOID,
	AZANDE_COMMAND_TYPE_BOOL,
	AZANDE_COMMAND_TYPE_ENUM,
	AZANDE_COMMAND_TYPE_INT,
	AZANDE_COMMAND_TYPE_DOUBLE,
	AZANDE_COMMAND_TYPE_TEXT,
}AZANDE_COMMAND_TYPE;

typedef void (*AZANDE_VOID_VOID_FUNC_PTR_T)(void);

typedef struct
{
	unsigned int id;
	const __FLASH_DIRECTIVE char* xml;
	AZANDE_COMMAND_TYPE type;
	AZANDE_VOID_VOID_FUNC_PTR_T cmd_function;
}AZANDE_COMMAND;

typedef enum
{
	AZANDE_EVENT_TYPE_BOOL,
	AZANDE_EVENT_TYPE_INT,
	AZANDE_EVENT_TYPE_DOUBLE,
	AZANDE_EVENT_TYPE_TEXT,
}AZANDE_EVENT_TYPE;

typedef struct
{
	unsigned int id;
	const __FLASH_DIRECTIVE char* xml;
	AZANDE_EVENT_TYPE type;
}AZANDE_EVENT;



//==============================================================================
//--------------------------- Public Functions ---------------------------------
//==============================================================================

void Azande__Init(const __FLASH_DIRECTIVE char* text);
void Azande__ReadStream(void);

void Azande__Clear();
void Azande__Notify();

void Azande__SendEventBool(const __FLASH_DIRECTIVE AZANDE_EVENT* event_ptr, uint8_t isTrue);
void Azande__SendEventInt32(const __FLASH_DIRECTIVE AZANDE_EVENT* event_ptr, int32_t value);
void Azande__SendEventDouble(const __FLASH_DIRECTIVE AZANDE_EVENT* event_ptr, double value);
void Azande__SendEventText(const __FLASH_DIRECTIVE AZANDE_EVENT* event_ptr, char* text);

void Azande__AddEvent(const __FLASH_DIRECTIVE AZANDE_EVENT* event_ptr);
void Azande__AddCommand(const __FLASH_DIRECTIVE AZANDE_COMMAND* cmd_ptr);


#endif /* AZANDE_H_ */
