/*
 * AzandeUserMacros.h
 *
 * Author : Anders Zeijlon
 */ 

#ifndef AZANDEUSERMACROS_H_
#define AZANDEUSERMACROS_H_


// Common macros ------------------------------------------
#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)
#define AZANDE_PROTOCOL_VERSION_MAJOR "1"
#define AZANDE_PROTOCOL_VERSION_MINOR "1"


// Target Info ------------------------------------------
#define define_target_info(VAR_NAME, TARGET, SW_VERSION, DATE, NOTE)\
static const __FLASH_DIRECTIVE char VAR_NAME[] =  "<Info><P><Mj>" AZANDE_PROTOCOL_VERSION_MAJOR "</Mj><Mi>" AZANDE_PROTOCOL_VERSION_MINOR "</Mi></P><N>" TARGET "</N><V>" SW_VERSION "</V><D>" DATE "</D><Nt>" NOTE "</Nt></Info>";\


// void Command ------------------------------------------
#define define_void_command(NAME, TITLE, GROUP, FUNCTION)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] =  "<F><C><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr></C></F>";\
static void FUNCTION(void);\
static const __FLASH_DIRECTIVE AZANDE_COMMAND NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_COMMAND_TYPE_VOID,\
	(AZANDE_VOID_VOID_FUNC_PTR_T)FUNCTION\
};\
static const __FLASH_DIRECTIVE AZANDE_COMMAND* const NAME = & NAME##_FEATURE_OBJ;



// Bool Command ------------------------------------------
#define define_bool_command(NAME, TITLE, GROUP, FUNCTION, TRUE_TXT, FALSE_TXT)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] =  "<F><Cb><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr><B1>" TRUE_TXT "</B1><B0>" FALSE_TXT "</B0></Cb></F>";\
static void FUNCTION(uint8_t);\
static const __FLASH_DIRECTIVE AZANDE_COMMAND NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_COMMAND_TYPE_BOOL,\
	(AZANDE_VOID_VOID_FUNC_PTR_T)FUNCTION\
};\
static const __FLASH_DIRECTIVE AZANDE_COMMAND* const NAME = & NAME##_FEATURE_OBJ;


// Int Command ------------------------------------------
#define define_int_command(NAME, TITLE, GROUP, FUNCTION, UNIT, MIN, MAX)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] =  "<F><Ci><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr><U>" UNIT "</U><Mn>" #MIN "</Mn><Mx>" #MAX "</Mx></Ci></F>";\
static void FUNCTION(int32_t);\
static const __FLASH_DIRECTIVE AZANDE_COMMAND NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_COMMAND_TYPE_INT,\
	(AZANDE_VOID_VOID_FUNC_PTR_T)FUNCTION\
};\
static const __FLASH_DIRECTIVE AZANDE_COMMAND* const NAME = & NAME##_FEATURE_OBJ;


// Enum Command ------------------------------------------
#define define_enum_item(VALUE, TXT)"<p><i>" #VALUE "</i><n>" TXT "</n></p>"

#define define_enum_command(NAME, TITLE, GROUP, FUNCTION, VALUES)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] = "<F><Ce><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr><ps>" VALUES "</ps></Ce></F>";\
static void FUNCTION(int32_t);\
static const __FLASH_DIRECTIVE AZANDE_COMMAND NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_COMMAND_TYPE_ENUM,\
	(AZANDE_VOID_VOID_FUNC_PTR_T)FUNCTION\
};\
static const __FLASH_DIRECTIVE AZANDE_COMMAND* const NAME = & NAME##_FEATURE_OBJ;




// Double Command ------------------------------------------
#define define_double_command(NAME, TITLE, GROUP, FUNCTION, UNIT, MIN, MAX)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] =  "<F><Cd><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr><U>" UNIT "</U><Mn>" #MIN "</Mn><Mx>" #MAX "</Mx></Cd></F>";\
static void FUNCTION(double);\
static const __FLASH_DIRECTIVE AZANDE_COMMAND NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_COMMAND_TYPE_DOUBLE,\
	(AZANDE_VOID_VOID_FUNC_PTR_T)FUNCTION\
};\
static const __FLASH_DIRECTIVE AZANDE_COMMAND* const NAME = & NAME##_FEATURE_OBJ;


// Text Command ------------------------------------------
#define define_text_command(NAME, TITLE, GROUP, FUNCTION, MAX_LEN)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] =  "<F><Ct><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr><Mx>" #MAX_LEN "</Mx></Ct></F>";\
static void FUNCTION(char*, unsigned int);\
static const __FLASH_DIRECTIVE AZANDE_COMMAND NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_COMMAND_TYPE_TEXT,\
	(AZANDE_VOID_VOID_FUNC_PTR_T)FUNCTION\
};\
static const __FLASH_DIRECTIVE AZANDE_COMMAND* const NAME = & NAME##_FEATURE_OBJ;


// Bool Event ------------------------------------------
#define define_bool_event(NAME, TITLE, GROUP, TRUE_TXT, FALSE_TXT)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] =  "<F><Eb><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr><B1>" TRUE_TXT "</B1><B0>" FALSE_TXT "</B0></Eb></F>";\
static const __FLASH_DIRECTIVE AZANDE_EVENT NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_EVENT_TYPE_BOOL\
};\
static const __FLASH_DIRECTIVE AZANDE_EVENT* const NAME = & NAME##_FEATURE_OBJ;


// Int Event ------------------------------------------
#define define_int_event(NAME, TITLE, GROUP, UNIT, MIN, MAX, FORMAT)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] =  "<F><Ei><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr><U>" UNIT "</U><Mn>" #MIN "</Mn><Mx>" #MAX "</Mx><Vf>" FORMAT "</Vf></Ei></F>";\
static const __FLASH_DIRECTIVE AZANDE_EVENT NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_EVENT_TYPE_INT\
};\
static const __FLASH_DIRECTIVE AZANDE_EVENT* const NAME = & NAME##_FEATURE_OBJ;


// Double Event ------------------------------------------
#define define_double_event(NAME, TITLE, GROUP, UNIT, MIN, MAX, FORMAT)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] =  "<F><Ed><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr><U>" UNIT "</U><Mn>" #MIN "</Mn><Mx>" #MAX "</Mx><Vf>" FORMAT "</Vf></Ed></F>";\
static const __FLASH_DIRECTIVE AZANDE_EVENT NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_EVENT_TYPE_DOUBLE\
};\
static const __FLASH_DIRECTIVE AZANDE_EVENT* const NAME = & NAME##_FEATURE_OBJ;


// Text Event ------------------------------------------
#define define_text_event(NAME, TITLE, GROUP, MAX_LEN)\
static const __FLASH_DIRECTIVE char AZANDE_XML_STRING_##NAME[] =  "<F><Et><I>" TOSTRING(__LINE__) "</I><Is>100</Is><N>" TITLE "</N><Gr>0" #GROUP "</Gr><Mx>" #MAX_LEN "</Mx></Et></F>";\
static const __FLASH_DIRECTIVE AZANDE_EVENT NAME##_FEATURE_OBJ =\
{\
	__LINE__,\
	AZANDE_XML_STRING_##NAME,\
	AZANDE_EVENT_TYPE_TEXT\
};\
static const __FLASH_DIRECTIVE AZANDE_EVENT* const NAME = & NAME##_FEATURE_OBJ;



#endif /* AZANDEUSERMACROS_H_ */
